// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DPF_H
#define DPF_H

#include <dfm-framework/backtrace/backtrace.h>
#include <dfm-framework/event/event.h>
#include <dfm-framework/lifecycle/lifecycle.h>
#include <dfm-framework/listener/listener.h>
#include <dfm-framework/log/framelogmanager.h>

#endif   // DPF_H
